function NMClab(method)

% NMClab(joint)
%
% A Graphical User Interface (GUI) to show the mechanical response
%  of a (linear) model of a human joint during posture control. The
%  parameter joint is optional, possible values:
%  'shoulder' (default), 'wrist', or 'ankle'.
%
% The model includes o.a. afferent feedback (proprioceptive reflexes) for
% position, velocity, acceleration (muscle spindles) and force (Golgi
% Tendon Organs, GTO).
%
% The model includes:
% - skeletal model
%       mass, passive visco-elasticity          m, bq, kq
% - muscle model
%       lumped visco-elasticity (co-contracion) b, k
%       series elastic element [optional]       kse
%       activation dynamics                     F0, Bt
% - afferent feedback model
%       feedback gains                          kp, kv, ka, kf
%       neural time delay                       Td
% - contact dynamics [optional]
%       contact visco-elasticity                bc, kc
% - environment model [optional]
%       mass, damping & stiffness               me, be, ke
%
% The performance, Jx, is normalized and defined as:
%    Jx=var(x)=|Hfx|^2 (see Schouten et al. 2001, Biol Cyber 84;143-152.)
%
% version 0.91: pre-release, February 29, 2008
% Copyright (C) 2007-2008, Alfred C. Schouten
% This software and model may be used, copied, or redistributed as long as
% it is not sold and this copyright notice is reproduced on each copy made.
% This model is provided as is, without any express or implied warranties
% whatsoever. 
%
% Please acknowledge the use of this model and software in any publications
% by refering to the article: 
%  "NMClab, a model to assess the contributions of muscle visco-elasticity
%  and afferent feedback to joint dynamics."
%  Schouten, Mugge, van der Helm. J Biomech, 2008, accepted.
%
% Alfred C. Schouten
% Delft Laboratory for NeuroMuscular Control
% Department of (Bio)Mechanical Engineering
% Delft University of Technology
%
% Mekelweg 2
% 2628 CD Delft
% The Netherlands
% e-mail: A.C.Schouten@tudelft.nl
% url:    www.3me.tudelft.nl/nmc

% version February 29, 2008
%
% todo:
%    - check necessary toolboxes
%    - step/impulse response are now hidden for unstable systems
%    - uipanels have absolute FontUnits (relative does not work properly, Matlab bug?)
%    - IRF of Hxa? (and what if Hxa is anticausal?)

%% check matlab version
mv=version;
if str2double(mv(1))<7
    error('NMClab requires Matlab 7 or higher')
end

%% shared variables (all structures)
% p: model parameters (depend on joint, e.g. muscle stiffness)
% q: model settings (depend on joint, e.g. frequency vector, axes limits)
% h: figure handles
% J: performance parameters
% pn: current parameter settings
persistent p q h J pn

%% normal case
if nargin==0
    method='buildup';
    joint='shoulder';
end
if strcmpi(method,'shoulder')
    joint='shoulder';
    method='buildup';
elseif strcmpi(method,'wrist')
    joint='wrist';
    method='buildup';
elseif strcmpi(method,'ankle')
    joint='ankle';
    method='buildup';
end

%% the cases
switch lower(method)
    case 'loaddata'
        [filename, pathname] = uigetfile('NMCdata.mat', 'Pick a file');
        htmp=NaN; %useless trick
        load([pathname filename]);
        % write parameters to sliders!!
        set(h.m,'Value',pn.m);
        set(h.bq,'Value',pn.bq);
        set(h.kq,'Value',pn.kq);
        set(h.b,'Value',pn.b);
        set(h.k,'Value',pn.k);
        if htmp.se~=get(h.check_se,'Value')
            set(h.check_se,'Value',htmp.se);
            % switch Series Elastic element 'on' or 'off'
            togglestr=[h.kse h.kseT];
            if get(h.check_se,'Value')==1
                set(togglestr,'Enable','On')
                set(h.kse,'Value',pn.kse);
            else
                set(togglestr,'Enable','Off')
            end
        end
        if htmp.c~=get(h.check_c,'Value')
            set(h.check_c,'Value',htmp.c);
            % switch Contact dynamics 'on' or 'off'
            togglestr=[h.bc h.bcT h.kc h.kcT];
            if get(h.check_c,'Value')==1
                set(togglestr,'Enable','On')
                set(h.bc,'Value',pn.bc);
                set(h.kc,'Value',pn.kc);
            else
                set(togglestr,'Enable','Off')
            end
        end
        set(h.me,'Value',pn.me);
        set(h.be,'Value',pn.be);
        set(h.ke,'Value',pn.ke);
        set(h.F0,'Value',pn.F0);
        set(h.Beta,'Value',pn.Bt);
        set(h.td,'Value',pn.Td);
        set(h.kp,'Value',pn.kp);
        set(h.kv,'Value',pn.kv);
        set(h.ka,'Value',pn.ka);
        set(h.kf,'Value',pn.kf);
        if htmp.e~=get(h.check_e,'Value')
            set(h.check_e,'Value',htmp.e);
        end

        %         % reset hold
        %         NMClab('hold')
        % evaluate model
        NMClab('togglee'); % includes NMC('evalmodel')
    case 'savedata'
        [filename, pathname] = uiputfile('NMCdata.mat', 'Save NMCdata as');
        htmp.se=get(h.check_se,'Value');
        htmp.e=get(h.check_e,'Value');
        htmp.c=get(h.check_c,'Value');
        save([pathname filename],'pn','htmp');
    case 'savefig'
        % implemented as is
        [filename, pathname] = uiputfile('NMCdata.fig', 'Save NMCfig as');
        saveas(h.fig,[pathname filename]);
    case 'about'
        % show about message, when 'about' is choosen in the 'help' menu
        about_txt={'NMClab, Januari 2008';' ';...
            'Copyright (C)';...
            'Alfred C. Schouten';...
            'e-mail: A.C.Schouten@tudelft.nl';' ';...
            'Delft University of Technology';...
            'Department of (Bio)Mechanical Engineering';...
            'Laboratory for neuromuscular control'};
        msgbox(about_txt,'About');
    case 'help'
        % show help message, when 'help' is choosen in the 'help' menu
        help_txt={'NMClab is interactive program to visualize the effect of afferent feedback on joint dynamics.';...
            'Type <help NMClab> for an overview of the parameters';' ';...
            'or contact';' ';...
            'Alfred C. Schouten';...
            'e-mail: A.C.Schouten@tudelft.nl';' ';...
            'Delft University of Technology';...
            'Department of (Bio)Mechanical Engineering';...
            'Delft Laboratory for Neuromuscular Control'};
        msgbox(help_txt,'Help');
    case 'buildup'
        % normal case: building up the gui %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        % initialize model parameters
        [p,q]=initpar(joint);
        % initpar is a nested function, see the end of this file

        % build figure with uicontrols %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        set(0,'Units','pixels')
        scrsz=get(0,'ScreenSize');
        % if a NMC-window already exists, use that one, else make a new one
        % it is not possible to have two parallel NMClab's (in one Matlab!)
        if isstruct(h)
            figure(h.fig);close(h.fig);figure(h.fig)
        else
            h.fig=figure;
        end

        set(h.fig,'MenuBar','none',...
            'Name',['NMClab:  ' joint],...
            'NumberTitle','off',...
            'Units','Pixels',...
            'DoubleBuffer','on',...
            'Position',[11 40 scrsz(3)-20 scrsz(4)-100]);

        % variable FontSize
        fsz_n=0.08; % normal, 768x1024 results in approx 14pt
        fsz_l=0.12; % large, 768x1024 results in approx 19pt

        % menu bar, own design
        h.menu = uimenu('Parent',h.fig,...
            'Label','File');
        h.loaddata = uimenu('Parent',h.menu,...
            'Label','Load data',...
            'Callback',...
            'NMClab(''loaddata'')');
        h.savedata = uimenu('Parent',h.menu,...
            'Label','Save data',...
            'Callback',...
            'NMClab(''savedata'')');
        %         h.savefig = uimenu('Parent',h.menu,...
        %             'Label','Save figure',...
        %             'Callback',...
        %             'NMClab(''savefig'')');
        h.printsetup = uimenu('Parent',h.menu,...
            'Label','Print Setup',...
            'Callback',...
            'orient landscape, print -dsetup');
        h.print = uimenu('Parent',h.menu,...
            'Label','Print',...
            'Callback',...
            'orient landscape, print');
        h.menu2 = uimenu('Parent',h.fig,...
            'Label','Help');
        h.help = uimenu('Parent',h.menu2,...
            'Label','Help',...
            'Callback',...
            'NMClab(''help'')');
        h.about = uimenu('Parent',h.menu2,...
            'Label','About NMClab',...
            'Separator','on',...
            'Callback',...
            'NMClab(''about'')');

        % parameter frame
        h.par=uipanel('Parent',h.fig,...
            'Units','normalized',...
            'Position',[0.7 0.02 0.285 0.96]);
        % FRF frame
        h.frf=uipanel('Parent',h.fig,...
            'Units','normalized',...
            'Position',[0.015 0.02 0.67 0.86],...
            'FontUnits','pixels',...
            'FontSize',18,...
            'Title','Frequency response functions');
        % stability frame
        h.stb=uipanel('Parent',h.fig,...
            'Units','normalized',...
            'Position',[0.015 0.02 0.67 0.86],...
            'FontUnits','pixels',...
            'FontSize',18,...
            'Title','Stability',...
            'Visible','off');
        % model frame (drawing!)
        h.mod=uipanel('Parent',h.fig,...
            'Units','normalized',...
            'Position',[0.015 0.02 0.67 0.86],...
            'FontUnits','pixels',...
            'FontSize',18,...
            'Title','Model',...
            'Visible','off');
        % impulse/step response frame
        h.imp=uipanel('Parent',h.fig,...
            'Units','normalized',...
            'Position',[0.015 0.02 0.67 0.86],...
            'FontUnits','pixels',...
            'FontSize',18,...
            'Title','Impulse response',...
            'Visible','off');

        % buttons to choose frame
        h.butfrf = uicontrol('Parent',h.fig,...
            'Units','normalized',...
            'Position',[0.015 0.9 0.145 0.075],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','togglebutton',...
            'Value',1,...
            'String','FRF',...
            'FontUnits','normalized',...
            'FontSize',0.4,...
            'Callback','NMClab(''but_frf'')');
        h.butstb = uicontrol('Parent',h.fig,...
            'Units','normalized',...
            'Position',[0.19 0.9 0.145 0.075],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','togglebutton',...
            'String','stability',...
            'FontUnits','normalized',...
            'FontSize',0.4,...
            'Callback','NMClab(''but_stb'')');
        h.butmod = uicontrol('Parent',h.fig,...
            'Units','normalized',...
            'Position',[0.365 0.9 0.145 0.075],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','togglebutton',...
            'String','model',...
            'FontUnits','normalized',...
            'FontSize',0.4,...
            'Callback','NMClab(''but_mod'')');
        h.butimp = uicontrol('Parent',h.fig,...
            'Units','normalized',...
            'Position',[0.54 0.9 0.145 0.075],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','togglebutton',...
            'String','impulse',...
            'FontUnits','normalized',...
            'FontSize',0.4,...
            'Callback','NMClab(''but_imp'')');

        %% parameter frame
        %
        % default & hold buttons
        h.def = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.52 0.01 0.44 0.05],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','pushbutton',...
            'String','Default',...
            'FontUnits','normalized',...
            'FontSize',0.4,...
            'Callback','NMClab(''setdefault'')');
        h.hold = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.52 0.07  0.44 0.05],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','pushbutton',...
            'String','Hold',...
            'FontUnits','normalized',...
            'FontSize',0.4,...
            'Callback','NMClab(''hold'')');

        %stability text
        h.stab = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.01 0.44 0.05],...
            'BackgroundColor',[0.1 1 0.1],...
            'Style','text',...
            'FontUnits','normalized',...
            'FontSize',0.5);
        h.stabtot = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.07 0.44 0.05],...
            'BackgroundColor',[0.1 1 0.1],...
            'Style','text',...
            'FontUnits','normalized',...
            'FontSize',0.6,...
            'Visible','off');

        % muscle subframe
        h.frame_int = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.02 0.68 0.96 0.31],...
            'BackgroundColor',[0.7 0.7 0.8],...
            'Style','Frame');
        h.text_int = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.96 0.92 0.03],...
            'BackgroundColor',[0.7 0.7 0.8],...
            'HorizontalAlignment','left',...
            'Style','text',...
            'String','Intrinsic Muscle Model',...
            'FontUnits','normalized',...
            'FontSize',0.8);
        % mass/inertia
        h.mT = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.92 0.45 0.03],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','text',...
            'FontUnits','normalized',...
            'FontSize',0.8);
        h.m = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.895 0.45 0.025],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','slider',...
            'Min',p.m.min,...
            'Max',p.m.max,...
            'Value',p.m.norm,...
            'Callback','NMClab(''evalmodel'')');
        % passive dynamics
        h.bqT = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.860 0.45 0.03],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','text',...
            'FontUnits','normalized',...
            'FontSize',0.8);
        h.bq = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.835 0.45 0.025],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','slider',...
            'Min',p.bq.min,...
            'Max',p.bq.max,...
            'Value',p.bq.norm,...
            'Callback','NMClab(''evalmodel'')');
        h.kqT = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.800 0.45 0.03],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','text',...
            'FontUnits','normalized',...
            'FontSize',0.8);
        h.kq = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.775 0.45 0.025],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','slider',...
            'Min',p.kq.min,...
            'Max',p.kq.max,...
            'Value',p.kq.norm,...
            'Callback','NMClab(''evalmodel'')');
        % series elastic element (tendon)
        h.check_se = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.745 0.45 0.025],...
            'BackgroundColor',[0.7 0.8 0.7],...
            'Style','checkbox',...
            'String','Series Elastic',...
            'FontUnits','normalized',...
            'FontSize',0.8,...
            'Callback','NMClab(''togglese'')');
        h.kseT = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.71 0.45 0.03],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','text',...
            'Enable','Off',...
            'FontUnits','normalized',...
            'FontSize',0.8);
        h.kse = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.685 0.45 0.025],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','slider',...
            'Enable','Off',...
            'Min',p.kse.min,...
            'Max',p.kse.max,...
            'Value',p.kse.norm,...
            'Callback','NMClab(''evalmodel'')');
        % active dynamics
        h.bT = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.51 0.920 0.45 0.03],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','text',...
            'FontUnits','normalized',...
            'FontSize',0.8);
        h.b = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.51 0.895 0.45 0.025],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','slider',...
            'Min',p.b.min,...
            'Max',p.b.max,...
            'Value',p.b.norm,...
            'Callback','NMClab(''evalmodel'')');
        h.kT = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.51 0.860 0.45 0.03],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','text',...
            'FontUnits','normalized',...
            'FontSize',0.8);
        h.k = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.51 0.835 0.45 0.025],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','slider',...
            'Min',p.k.min,...
            'Max',p.k.max,...
            'Value',p.k.norm,...
            'Callback','NMClab(''evalmodel'')');
        % muscle activation dynamics
        h.F0T = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.51 0.77 0.45 0.03],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','text',...
            'FontUnits','normalized',...
            'FontSize',0.8);
        h.F0 = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.51 0.745 0.45 0.025],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','slider',...
            'Min',p.F0.min,...
            'Max',p.F0.max,...
            'Value',p.F0.norm,...
            'Callback','NMClab(''evalmodel'')');
        h.BetaT = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.51 0.71 0.45 0.03],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','text',...
            'FontUnits','normalized',...
            'FontSize',0.8);
        h.Beta = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.51 0.685 0.45 0.025],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','slider',...
            'Min',p.Bt.min,...
            'Max',p.Bt.max,...
            'Value',p.Bt.norm,...
            'Callback','NMClab(''evalmodel'')');

        % contact & environment subframe
        h.frame_env = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.02 0.42 0.96 0.25],...
            'BackgroundColor',[0.7 0.7 0.8],...
            'Style','Frame');
        h.text_env = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.64 0.92 0.03],...
            'BackgroundColor',[0.7 0.7 0.8],...
            'HorizontalAlignment','left',...
            'Style','text',...
            'String','Environment & Contact dynamics',...
            'FontUnits','normalized',...
            'FontSize',0.8);
        % environment
        h.check_e = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.605 0.45 0.025],...
            'BackgroundColor',[0.7 0.8 0.7],...
            'Style','checkbox',...
            'String','environment',...
            'FontUnits','normalized',...
            'FontSize',0.8,...
            'Callback','NMClab(''togglee'')');
        h.meT = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.57 0.45 0.03],...
            'BackgroundColor',[0.7 0.8 0.7],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','text',...
            'Enable','Off',...
            'FontUnits','normalized',...
            'FontSize',0.8);
        h.me = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.545 0.45 0.025],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','slider',...
            'Enable','Off',...
            'Min',p.me.min,...
            'Max',p.me.max,...
            'Value',p.me.norm,...
            'Callback','NMClab(''evalmodel'')');
        h.beT = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.51 0.45 0.03],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','text',...
            'Enable','Off',...
            'FontUnits','normalized',...
            'FontSize',0.8);
        h.be = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.485 0.45 0.025],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','slider',...
            'Enable','Off',...
            'Min',p.be.min,...
            'Max',p.be.max,...
            'Value',p.be.norm,...
            'Callback','NMClab(''evalmodel'')');
        h.keT = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.45 0.45 0.03],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','text',...
            'Enable','Off',...
            'FontUnits','normalized',...
            'FontSize',0.8);
        h.ke = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.425 0.45 0.025],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','slider',...
            'Enable','Off',...
            'Min',p.ke.min,...
            'Max',p.ke.max,...
            'Value',p.ke.norm,...
            'Callback','NMClab(''evalmodel'')');
        % contact
        h.check_c = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.51 0.605 0.45 0.025],...
            'BackgroundColor',[0.7 0.8 0.7],...
            'Style','checkbox',...
            'String','contact',...
            'FontUnits','normalized',...
            'FontSize',0.8,...
            'Callback','NMClab(''togglec'')');
        h.bcT = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.51 0.51 0.45 0.03],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','text',...
            'Enable','Off',...
            'FontUnits','normalized',...
            'FontSize',0.8);
        h.bc = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.51 0.485 0.45 0.025],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','slider',...
            'Enable','Off',...
            'Min',p.bc.min,...
            'Max',p.bc.max,...
            'Value',p.bc.norm,...
            'Callback','NMClab(''evalmodel'')');
        h.kcT = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.51 0.45 0.45 0.03],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','text',...
            'Enable','Off',...
            'FontUnits','normalized',...
            'FontSize',0.8);
        h.kc = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.51 0.425 0.45 0.025],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','slider',...
            'Enable','Off',...
            'Min',p.kc.min,...
            'Max',p.kc.max,...
            'Value',p.kc.norm,...
            'Callback','NMClab(''evalmodel'')');

        % reflexes subframe
        h.frame_ref = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.02 0.13 0.96 0.28],...
            'BackgroundColor',[0.7 0.7 0.8],...
            'Style','Frame');
        h.text_ref = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.38 0.92 0.03],...
            'BackgroundColor',[0.7 0.7 0.8],...
            'HorizontalAlignment','left',...
            'Style','text',...
            'String','Reflexive Model',...
            'FontUnits','normalized',...
            'FontSize',0.8);
        % reflex gains
        h.kpT = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.34 0.45 0.03],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','text',...
            'FontUnits','normalized',...
            'FontSize',0.8);
        h.kp = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.315 0.45 0.025],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','slider',...
            'Min',p.kp.min,...
            'Max',p.kp.max,...
            'Value',p.kp.norm,...
            'Callback','NMClab(''evalmodel'')');
        h.kvT = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.28 0.45 0.03],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','text',...
            'FontUnits','normalized',...
            'FontSize',0.8);
        h.kv = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.255 0.45 0.025],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','slider',...
            'Min',p.kv.min,...
            'Max',p.kv.max,...
            'Value',p.kv.norm,...
            'Callback','NMClab(''evalmodel'')');
        h.kaT = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.22 0.45 0.03],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','text',...
            'FontUnits','normalized',...
            'FontSize',0.8);
        h.ka = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.195 0.45 0.025],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','slider',...
            'Min',p.ka.min,...
            'Max',p.ka.max,...
            'Value',p.ka.norm,...
            'Callback','NMClab(''evalmodel'')');
        h.kfT = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.16 0.45 0.03],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','text',...
            'TooltipString','Feel the force',...
            'FontUnits','normalized',...
            'FontSize',0.8);
        h.kf = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.04 0.135 0.45 0.025],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','slider',...
            'TooltipString','Feel the force',...
            'Min',p.kf.min,...
            'Max',p.kf.max,...
            'Value',p.kf.norm,...
            'Callback','NMClab(''evalmodel'')');
        % reflex delays
        h.tdT = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.51 0.34 0.45 0.03],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','text',...
            'FontUnits','normalized',...
            'FontSize',0.8);
        h.td = uicontrol('Parent',h.par,...
            'Units','normalized',...
            'Position',[0.51 0.315 0.45 0.025],...
            'BackgroundColor',[0.7 0.8 0.8],...
            'Style','slider',...
            'Min',p.Td.min,...
            'Max',p.Td.max,...
            'Value',p.Td.norm,...
            'Callback','NMClab(''evalmodel'')');

        %% FRF WINDOW
        % gain and phase diagrams of the mechnical admittance
        h.axGfx = axes('Parent',h.frf,...
            'XTick',q.ax.freq.tick,...
            'XTickLabel',[],...
            'Xscale','log',...
            'Yscale','log',...
            'XLim',q.ax.freq.val,...
            'YLim',q.ax.frf_ma.val,...
            'FontUnits','normalized',...
            'Units','normalized',...
            'Position',[0.1 0.5 0.38 0.38]);
        set(h.axGfx,'Ylabel',text('String',q.ax.frf_ma.ylabel,'FontUnits','normalized','FontSize',fsz_n));
        set(h.axGfx,'Title',text('String',q.ax.frf_ma.title,'FontUnits','normalized','FontSize',fsz_l));
        h.axPfx = axes('Parent',h.frf,...
            'XTick',q.ax.freq.tick,...
            'XTickLabel',q.ax.freq.label,...
            'YTick',[-180 -90 0 90 180],...
            'Xscale','log',...
            'XLim',q.ax.freq.val,...
            'YLim',[-180,180],...
            'FontUnits','normalized',...
            'Units','normalized',...
            'Position',[0.1 0.1 0.38 0.38]);
        set(h.axPfx,'Xlabel',text('String','frequency [Hz]','FontUnits','normalized','FontSize',fsz_n))
        set(h.axPfx,'Ylabel',text('String','phase [deg]','FontUnits','normalized','FontSize',fsz_n))
        % lines: mechanical admittance (b- thick), hold (b-)
        h.lineGfx = line('Parent',h.axGfx,'Color','b','LineStyle','-','LineWidth',2);
        h.linePfx = line('Parent',h.axPfx,'Color','b','LineStyle','-','LineWidth',2);
        h.lineGfx2 = line('Parent',h.axGfx,'Color','b','LineStyle','-');
        h.linePfx2 = line('Parent',h.axPfx,'Color','b','LineStyle','-');
        % lines: total admittance (r: thick), total hold (r:)
        % hidden lines, default starts without environment!
        h.lineGdx = line('Parent',h.axGfx,'Color','r','LineStyle',':','LineWidth',2,'Visible','Off');
        h.linePdx = line('Parent',h.axPfx,'Color','r','LineStyle',':','LineWidth',2,'Visible','Off');
        h.lineGdx2 = line('Parent',h.axGfx,'Color','r','LineStyle',':','Visible','Off');
        h.linePdx2 = line('Parent',h.axPfx,'Color','r','LineStyle',':','Visible','Off');

        % figure legend
        h.legend=legend(h.axPfx,[h.lineGfx,h.lineGfx2,h.lineGdx,h.lineGdx2],...
            {'joint dynamics','hold joint','total dynamics','hold total'});
        set(h.legend,'Box','off');
        h.legendchildren=get(h.legend,'Children');

        % gain and phase diagrams of the reflexive impedance
        h.axGxa = axes('Parent',h.frf,...
            'XTick',q.ax.freq.tick,...
            'XTickLabel',[],...
            'Xscale','log',...
            'Yscale','log',...
            'XLim',q.ax.freq.val,...
            'YLim',q.ax.frf_ri.val,...
            'FontUnits','normalized',...
            'Units','normalized',...
            'Position',[0.6 0.5 0.38 0.38]);
        set(h.axGxa,'Ylabel',text('String',q.ax.frf_ri.ylabel,'FontUnits','normalized','FontSize',fsz_n));
        set(h.axGxa,'Title',text('String',q.ax.frf_ri.title,'FontUnits','normalized','FontSize',fsz_l));
        h.axPxa = axes('Parent',h.frf,...
            'XTick',q.ax.freq.tick,...
            'XTickLabel',q.ax.freq.label,...
            'YTick',(-360:90:360),...
            'Xscale','log',...
            'XLim',q.ax.freq.val,...
            'YLim',[-180,180],...
            'FontUnits','normalized',...
            'Units','normalized',...
            'Position',[0.6 0.1 0.38 0.38]);
        set(h.axPxa,'Xlabel',text('String','frequency [Hz]','FontUnits','normalized','FontSize',fsz_n))
        set(h.axPxa,'Ylabel',text('String','phase [deg]','FontUnits','normalized','FontSize',fsz_n))
        % lines: reflexive impedance (b- thick), hold (b-)
        h.lineGxa = line('Parent',h.axGxa,'Color','b','LineStyle','-','LineWidth',2);
        h.linePxa = line('Parent',h.axPxa,'Color','b','LineStyle','-','LineWidth',2);
        h.lineGxa2 = line('Parent',h.axGxa,'Color','b','LineStyle','-');
        h.linePxa2 = line('Parent',h.axPxa,'Color','b','LineStyle','-');

        % performance text
        h.JxhT = text('Parent',h.axGfx,...
            'Position',[10^(0.9*log10(q.ax.freq.val(1))+0.1*log10(q.ax.freq.val(2))) 2*q.ax.frf_ma.val(1)],...
            'Color','b',...
            'FontUnits','normalized',...
            'FontSize',fsz_n);
        h.JxtT = text('Parent',h.axGfx,...
            'Position',[10^mean(log10(q.ax.freq.val)) 2*q.ax.frf_ma.val(1)],...
            'Color','r',...
            'Visible','Off',...
            'FontUnits','normalized',...
            'FontSize',fsz_n);
        %h.text = text('Parent',h.axGfx,'Position',[1.1,0.000022],'Color','k','Interpreter','TeX');
        %h.text2 = text('Parent',h.axGfx,'Position',[1.1,0.000015],'Color','k','Interpreter','TeX');

        %% STABILITY WINDOW
        % Nyquist plot
        h.axNyq = axes('Parent',h.stb,...
            'XLim',[-3 3],...
            'YLim',[-3 3],...
            'FontUnits','normalized',...
            'Units','normalized',...
            'Position',[0.1 0.5 0.38 0.38]);
        set(h.axNyq,'Xlabel',text('String','real H_{ol}','FontUnits','normalized','FontSize',fsz_n));
        set(h.axNyq,'Ylabel',text('String','imag H_{ol}','FontUnits','normalized','FontSize',fsz_n));
        set(h.axNyq,'Title',text('String','Nyquist open-loop','FontUnits','normalized','FontSize',fsz_l));
        % reference frame
        line('Parent',h.axNyq,'Color','k','Marker','.','XData',-1,'YData',0,'MarkerSize',15)
        line('Parent',h.axNyq,'Color','k','LineStyle','--','XData',[-10 10],'YData',[0 0])
        line('Parent',h.axNyq,'Color','k','LineStyle','--','XData',[0 0],'YData',[-10 10])
        % lines: OL
        h.lineNyq = line('Parent',h.axNyq,'Color','b','LineStyle','-','LineWidth',2);
        h.lineNyqt = line('Parent',h.axNyq,'Color','r','LineStyle',':','LineWidth',2,'Visible','Off');
        % M-circles in nyquist diagram (not yet)

        % gain and phase diagrams Open-Loop
        h.axGol = axes('Parent',h.stb,...
            'XTick',q.ax.freq.tick,...
            'XTickLabel',[],...
            'Xscale','log',...
            'Yscale','log',...
            'XLim',q.ax.freq.val,...
            'YLim',[1e-1,1e1],...
            'FontUnits','normalized',...
            'Units','normalized',...
            'Position',[0.6 0.5 0.38 0.38]);
        set(h.axGol,'Ylabel',text('String','gain [-]','FontUnits','normalized','FontSize',fsz_n));
        set(h.axGol,'Title',text('String','Bode open-loop','FontUnits','normalized','FontSize',fsz_l));
        h.axPol = axes('Parent',h.stb,...
            'XTick',q.ax.freq.tick,...
            'XTickLabel',q.ax.freq.label,...
            'YTick',(-360:90:90),...
            'Xscale','log',...
            'XLim',q.ax.freq.val,...
            'YLim',[-360,0],...
            'FontUnits','normalized',...
            'Units','normalized',...
            'Position',[0.6 0.1 0.38 0.38]);
        set(h.axPol,'Xlabel',text('String','frequency [Hz]','FontUnits','normalized','FontSize',fsz_n))
        set(h.axPol,'Ylabel',text('String','phase [deg]','FontUnits','normalized','FontSize',fsz_n))
        % reference lines
        line('Parent',h.axGol,'Color','k','LineStyle','--','XData',q.ax.freq.val,'YData',[1 1])
        line('Parent',h.axPol,'Color','k','LineStyle','--','XData',q.ax.freq.val,'YData',[-180 -180])
        % lines:
        h.lineGol = line('Parent',h.axGol,'Color','b','LineStyle','-','LineWidth',2);
        h.linePol = line('Parent',h.axPol,'Color','b','LineStyle','-','LineWidth',2);
        h.lineGoldx = line('Parent',h.axGol,'Color','r','LineStyle',':','LineWidth',2,'Visible','Off');
        h.linePoldx = line('Parent',h.axPol,'Color','r','LineStyle',':','LineWidth',2,'Visible','Off');

        %% MODEL WINDOW
        % drawing with the model!
        if exist(q.ModName,'file')
            % axes and image
            h.axMDL = axes('Parent',h.mod,...
                'Units','normalized',...
                'Position',[0.05 0.05 0.9 0.9],...
                'YDir','reverse',...
                'DataAspectRatio',[1 1 1]);
            h.mdl=image('Parent',h.axMDL);
            axis off
            % load and show image
            mdl=imread(q.ModName);
            set(h.mdl,'cdata',mdl);
            tmp=get(h.mod,'Backgroundcolor');
            colormap([0 0 0;tmp])
        end

        %% IMPULSE WINDOW
        h.axIMP = axes('Parent',h.imp,...
            'XLim',q.ax.time.val,...
            'XTickLabel',[],...
            'YLim',q.ax.imp.val,...
            'FontUnits','normalized',...
            'Units','normalized',...
            'Position',[0.1 0.5 0.38 0.38]);
        set(h.axIMP,'Ylabel',text('String',q.ax.imp.ylabel,'FontUnits','normalized','FontSize',fsz_n));
        set(h.axIMP,'Title',text('String',q.ax.imp.title,'FontUnits','normalized','FontSize',fsz_l));
        h.axST = axes('Parent',h.imp,...
            'XLim',q.ax.time.val,...
            'YLim',q.ax.step.val,...
            'FontUnits','normalized',...
            'Units','normalized',...
            'Position',[0.1 0.1 0.38 0.38]);
        set(h.axST,'Ylabel',text('String',q.ax.step.ylabel,'FontUnits','normalized','FontSize',fsz_n));
        set(h.axST,'Xlabel',text('String','time [s]','FontUnits','normalized','FontSize',fsz_n))
        %         h.axIMPxa = axes('Parent',h.imp,...
        %             'XLim',[0 2],...
        %             'YLim',[-10 10],...
        %             'Units','normalized',...
        %             'Position',[0.6 0.5 0.38 0.38]);
        %         h.axSTe = axes('Parent',h.imp,...
        %             'XLim',[0 2],...
        %             'YLim',[-10 10],...
        %             'Units','normalized',...
        %             'Position',[0.6 0.1 0.38 0.38]);
        % lines
        h.lineImp = line('Parent',h.axIMP,'Color','b','LineStyle','-','LineWidth',2);
        h.lineStp = line('Parent',h.axST,'Color','b','LineStyle','-','LineWidth',2);
        %h.lineImpe = line('Parent',h.axIMPxa,'Color','b','LineStyle','-','LineWidth',2);
        %h.lineStpe = line('Parent',h.axSTe,'Color','b','LineStyle','-','LineWidth',2);
        % lines: total (r-- invisible~!)
        % hidden lines, normally started without environment!
        h.lineImpt = line('Parent',h.axIMP,'Color','r','LineStyle',':','LineWidth',2,'Visible','Off');
        h.lineStpt = line('Parent',h.axST,'Color','r','LineStyle',':','LineWidth',2,'Visible','Off');

        % set to default (not strictly necessary!?)
        % reset hold button
        NMClab('hold')
        % evaluate model (for the first time!)
        NMClab('evalmodel')
    case 'evalmodel'
        % calculate response and performance, draw both

        % read parameters from slider %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        pn.m=get(h.m,'Value');
        pn.bq=get(h.bq,'Value');
        pn.kq=get(h.kq,'Value');
        pn.b=get(h.b,'Value');
        pn.k=get(h.k,'Value');
        if get(h.check_se,'Value')==1
            pn.kse=get(h.kse,'Value');
        else
            pn.kse=inf;
        end
        if get(h.check_c,'Value')==1
            pn.bc=get(h.bc,'Value');
            pn.kc=get(h.kc,'Value');
        else
            pn.bc=[];
            pn.kc=[];
        end
        pn.me=get(h.me,'Value');
        pn.be=get(h.be,'Value');
        pn.ke=get(h.ke,'Value');
        pn.F0=get(h.F0,'Value');
        pn.Bt=get(h.Beta,'Value');
        pn.Td=get(h.td,'Value');
        pn.kp=get(h.kp,'Value');
        pn.kv=get(h.kv,'Value');
        pn.ka=get(h.ka,'Value');
        pn.kf=get(h.kf,'Value');

        % write parameters to textboxes
        set(h.mT,'String',['m=' num2str(pn.m) p.m.unit])
        set(h.bqT,'String',['bq=' num2str(pn.bq) p.bq.unit])
        set(h.kqT,'String',['kq=' num2str(pn.kq) p.kq.unit])
        set(h.bT,'String',['b=' num2str(pn.b) p.b.unit])
        set(h.kT,'String',['k=' num2str(pn.k) p.k.unit])
        if get(h.check_se,'Value')==1
            set(h.kseT,'String',['kse=' num2str(pn.kse/p.kse.gain) p.kse.unit])
        else
            set(h.kseT,'String','')
        end
        if get(h.check_c,'Value')==1
            set(h.bcT,'String',['bc=' num2str(pn.bc) p.bc.unit])
            set(h.kcT,'String',['kc=' num2str(pn.kc/p.kc.gain) p.kc.unit])
        else
            set(h.bcT,'String','')
            set(h.kcT,'String','')
        end
        if get(h.check_e,'Value')==1
            set(h.meT,'String',['me=' num2str(pn.me) p.me.unit])
            set(h.beT,'String',['be=' num2str(pn.be) p.be.unit])
            set(h.keT,'String',['ke=' num2str(pn.ke) p.ke.unit])
        else
            set(h.meT,'String','')
            set(h.beT,'String','')
            set(h.keT,'String','')
        end
        set(h.F0T,'String',['F_0=' num2str(pn.F0) p.F0.unit])
        set(h.BetaT,'String',['\beta=' num2str(pn.Bt) ''])
        set(h.tdT,'String',['Td=' num2str(pn.Td/p.Td.gain) p.Td.unit])
        set(h.kpT,'String',['kp=' num2str(pn.kp) p.kp.unit])
        set(h.kvT,'String',['kv=' num2str(pn.kv) p.kv.unit])
        set(h.kaT,'String',['ka=' num2str(pn.ka) p.ka.unit])
        set(h.kfT,'String',['kf=' num2str(pn.kf) p.kf.unit])

        % calculate response
        [H,pn,Jn]=NMC_tf(pn,q.fr);
        % calculate impulse response
        [Hirf]=NMC_tf(pn,q.fr_imp);
        H_imp=Hirf.Hfx.*exp(-pi*q.fr_imp*1j); % delay with 0.5 second
        Ht_imp=Hirf.Hdx.*exp(-pi*q.fr_imp*1j); % delay with 0.5 second
        himpulse=real(q.Fs*ifft([H_imp conj(fliplr(H_imp(2:end-1)))]));
        himpulset=real(q.Fs*ifft([Ht_imp conj(fliplr(Ht_imp(2:end-1)))]));
        % step response is integral of the impulse response
        hstep=cumsum(himpulse)./q.Fs;
        hstept=cumsum(himpulset)./q.Fs;
        % d => e : Ht_imp.*He_imp.*filter 100 rad/s (+-16Hz)

        % FRF: draw line
        set(h.lineGfx,'XData',q.fr,'YData',abs(H.Hfx))
        set(h.linePfx,'XData',q.fr,'YData',unwrap(angle(H.Hfx))*180/pi)
        set(h.lineGxa,'XData',q.fr,'YData',abs(H.Hxa))
        set(h.linePxa,'XData',q.fr,'YData',unwrap(angle(H.Hxa))*180/pi)
        % writes TOTAL system (hidden or not)
        set(h.lineGdx,'XData',q.fr,'YData',abs(H.Hdx))
        set(h.linePdx,'XData',q.fr,'YData',unwrap(angle(H.Hdx))*180/pi)
        % calculate performance
        set(h.JxhT,'String',['J_{x} = ' num2str(round(Jn.xh/J.man*100)) ' %'])
        set(h.JxtT,'String',['J_{x} = ' num2str(round(Jn.xt/J.tot*100)) ' %'])

        % STABILITY: draw lines
        % open loop response, nyquist
        set(h.lineNyq,'XData',real(H.Holh),'YData',imag(H.Holh))
        set(h.lineNyqt,'XData',real(H.Holt),'YData',imag(H.Holt))
        % open loop response, bode
        set(h.lineGol,'XData',q.fr,'YData',abs(H.Holh))
        set(h.linePol,'XData',q.fr,'YData',unwrap(angle(H.Holh))*180/pi)
        set(h.lineGoldx,'XData',q.fr,'YData',abs(H.Holt))
        set(h.linePoldx,'XData',q.fr,'YData',unwrap(angle(H.Holt))*180/pi)

        % stab txt
        if H.unsh==0
            set(h.stab,'String','Human Stable','BackgroundColor',[0.1 1 0.1])
            %set(h.JxhT,'Visible','On')
        else
            set(h.stab,'String','Human Unstable','BackgroundColor',[1 0.1 0.1])
            %set(h.JxhT,'Visible','Off')
        end
        if H.unst==0
            set(h.stabtot,'String','Total Stable','BackgroundColor',[0.1 1 0.1])
            %set(h.JxtT,'Visible','On')
        else
            set(h.stabtot,'String','Total Unstable','BackgroundColor',[1 0.1 0.1])
            %set(h.JxtT,'Visible','Off')
        end

        % IMPULSE: drawline
        if H.unsh==0
            set(h.lineImp,'Xdata',q.t_imp,'YData',himpulse);
            set(h.lineStp,'Xdata',q.t_imp,'YData',hstep);
        else
            set(h.lineImp,'Xdata',NaN,'YData',NaN);
            set(h.lineStp,'Xdata',NaN,'YData',NaN);
        end
        if H.unst==0
            set(h.lineImpt,'Xdata',q.t_imp,'YData',himpulset);
            set(h.lineStpt,'Xdata',q.t_imp,'YData',hstept);
        else
            set(h.lineImpt,'Xdata',NaN,'YData',NaN);
            set(h.lineStpt,'Xdata',NaN,'YData',NaN);
        end
        %set(h.lineImpe,'Xdata',q.t_imp,'YData',himpulsee);
        %set(h.lineStpe,'Xdata',q.t_imp,'YData',hstepe);

    case 'hold'
        % -hold current setting and write (hold-)performance

        % read parameters from slider %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        ph.m=get(h.m,'Value');
        ph.bq=get(h.bq,'Value');
        ph.kq=get(h.kq,'Value');
        ph.b=get(h.b,'Value');
        ph.k=get(h.k,'Value');
        if get(h.check_se,'Value')==1
            ph.kse=get(h.kse,'Value');
        else
            ph.kse=inf;
        end
        if get(h.check_c,'Value')==1
            ph.bc=get(h.bc,'Value');
            ph.kc=get(h.kc,'Value');
        else
            ph.bc=[];
            ph.kc=[];
        end
        ph.me=get(h.me,'Value');
        ph.be=get(h.be,'Value');
        ph.ke=get(h.ke,'Value');
        ph.F0=get(h.F0,'Value');
        ph.Bt=get(h.Beta,'Value');
        ph.Td=get(h.td,'Value');
        ph.kp=get(h.kp,'Value');
        ph.kv=get(h.kv,'Value');
        ph.ka=get(h.ka,'Value');
        ph.kf=get(h.kf,'Value');

        % calculate response
        [H,ph,Jn]=NMC_tf(ph,q.fr);
        % write dashed line (is the 'hold' condition)
        set(h.lineGfx2,'XData',q.fr,'YData',abs(H.Hfx))
        set(h.linePfx2,'XData',q.fr,'YData',unwrap(angle(H.Hfx))*180/pi)
        set(h.lineGxa2,'XData',q.fr,'YData',abs(H.Hxa))
        set(h.linePxa2,'XData',q.fr,'YData',unwrap(angle(H.Hxa))*180/pi)
        set(h.lineGdx2,'XData',q.fr,'YData',abs(H.Hdx))
        set(h.linePdx2,'XData',q.fr,'YData',unwrap(angle(H.Hdx))*180/pi)
        % calculate  performance J.man
        J.man=Jn.xh;
        J.tot=Jn.xt;
        set(h.JxhT,'String',['J_{x} = ' num2str(round(J.man/J.man*100)) ' %'])
        set(h.JxtT,'String',['J_{x} = ' num2str(round(J.tot/J.tot*100)) ' %'])

    case 'setdefault'
        % reset to initial values
        tmp=fieldnames(p);
        for loop=1:length(tmp)
            pn.(tmp{loop})=p.(tmp{loop}).norm;
        end
        % write parameters to sliders!!
        set(h.m,'Value',pn.m);
        set(h.bq,'Value',pn.bq);
        set(h.kq,'Value',pn.kq);
        set(h.b,'Value',pn.b);
        set(h.k,'Value',pn.k);
        set(h.kse,'Value',pn.kse);
        set(h.bc,'Value',pn.bc);
        set(h.kc,'Value',pn.kc);
        set(h.me,'Value',pn.me);
        set(h.be,'Value',pn.be);
        set(h.ke,'Value',pn.ke);
        set(h.F0,'Value',pn.F0);
        set(h.Beta,'Value',pn.Bt);
        set(h.td,'Value',pn.Td);
        set(h.kp,'Value',pn.kp);
        set(h.kv,'Value',pn.kv);
        set(h.ka,'Value',pn.ka);
        set(h.kf,'Value',pn.kf);
        % reset hold
        NMClab('hold')
        % evaluate armmodel
        NMClab('evalmodel')
    case 'but_frf'
        % show the FRF window
        % set(h_butfrf,'value',0)
        set(h.butstb,'value',0)
        set(h.butmod,'value',0)
        set(h.butimp,'value',0)
        set(h.frf,'visible','on')
        set(h.stb,'visible','off')
        set(h.mod,'visible','off')
        set(h.imp,'visible','off')
    case 'but_stb'
        % show the STABILITY window
        set(h.butfrf,'value',0)
        % set(h_butstb,'value',0)
        set(h.butmod,'value',0)
        set(h.butimp,'value',0)
        set(h.frf,'visible','off')
        set(h.stb,'visible','on')
        set(h.mod,'visible','off')
        set(h.imp,'visible','off')
    case 'but_mod'
        % show the MODEL window
        set(h.butfrf,'value',0)
        set(h.butstb,'value',0)
        % set(h_butmod,'value',0)
        set(h.butimp,'value',0)
        set(h.frf,'visible','off')
        set(h.stb,'visible','off')
        set(h.mod,'visible','on')
        set(h.imp,'visible','off')
    case 'but_imp'
        % show the IMPULSE window
        set(h.butfrf,'value',0)
        set(h.butstb,'value',0)
        set(h.butmod,'value',0)
        % set(h_butimp,'value',0)
        set(h.frf,'visible','off')
        set(h.stb,'visible','off')
        set(h.mod,'visible','off')
        set(h.imp,'visible','on')

    case 'togglese'
        % switch Series Elastic element 'on' or 'off'
        togglestr=[h.kse h.kseT];
        if get(h.check_se,'Value')==1
            set(togglestr,'Enable','On')
        else
            set(togglestr,'Enable','Off')
        end
        % evaluate armmodel
        NMClab('evalmodel')
    case 'togglec'
        % switch Contact dynamics 'on' or 'off'
        togglestr=[h.bc h.bcT h.kc h.kcT];
        if get(h.check_c,'Value')==1
            set(togglestr,'Enable','On')
        else
            set(togglestr,'Enable','Off')
        end
        % evaluate armmodel
        NMClab('evalmodel')
    case 'togglee'
        % switch Environment 'on' or 'off'
        % note that lines are always drawn (hidden or not!)
        togglestr=[h.me h.meT h.be h.beT h.ke h.keT];
        if get(h.check_e,'Value')==1
            set(togglestr,'Enable','On')
            set(h.lineGdx,'Visible','On')
            set(h.linePdx,'Visible','On')
            set(h.lineGdx2,'Visible','On')
            set(h.linePdx2,'Visible','On')
            set(h.lineNyqt,'Visible','On')
            set(h.lineGoldx,'Visible','On')
            set(h.linePoldx,'Visible','On')
            set(h.JxtT,'Visible','On')
            set(h.lineImpt,'Visible','On')
            set(h.lineStpt,'Visible','On')
            set(h.stabtot,'Visible','On')
            set(h.legendchildren(1:6),'Visible','On')
        else
            set(togglestr,'Enable','Off')
            set(h.lineGdx,'Visible','Off')
            set(h.linePdx,'Visible','Off')
            set(h.lineGdx2,'Visible','Off')
            set(h.linePdx2,'Visible','Off')
            set(h.lineNyqt,'Visible','Off')
            set(h.lineGoldx,'Visible','Off')
            set(h.linePoldx,'Visible','Off')
            set(h.JxtT,'Visible','Off')
            set(h.lineImpt,'Visible','Off')
            set(h.lineStpt,'Visible','Off')
            set(h.stabtot,'Visible','Off')
            set(h.legendchildren(1:6),'Visible','Off')
        end
        % evaluate armmodel
        NMClab('evalmodel')
    otherwise
        error('Choose ''shoulder'', ''wrist'', or ''ankle'' as input')
end

%% NMC_tf
function [H,p,J]=NMC_tf(p,f)
% s-vector (LaPlace operator)
s=2*pi*1j*f;
s2=-4*pi^2*f.^2; % j^2=-1
% inertial and passive properties [Eq.A1]
Hi_inv=p.m*s2+p.bq*s+p.kq;
% intrinsic muscle properties (co-contraction) [Eq.A3]
Hce=p.k+p.b*s;
% muscle activation dynamics [Eq.A4]
a1=1./( (2*pi*p.F0)^2 );
a2=2*p.Bt./(2*pi*p.F0);
Hact=1./(a1*s2+a2*s+1);
% muscle spindle [Eq.A6]
Hms=(p.kp+p.kv*s+p.ka*s2).*exp(-p.Td*s);
% Golgi tendon organ [Eq.A7]
Hgto=p.kf*exp(-p.Td*s);
% environment [Eq.A13]
He_inv=p.me*s2+p.be*s+p.ke;
% the filter made by the GTO [Eq.1]
Hfiltgto=1./(1+Hgto.*Hact);
% intrinsic and reflexive feedback [Eq.2]
Hfb=Hce+(Hms-Hce.*Hgto).*Hfiltgto.*Hact;
% filter made by the SE (X=>Xse) [Eq.3]
Hfiltse=1./(1+Hfb./p.kse);
% joint admittance (F=>X) [Eq.4]
H.Hfx=1./(Hi_inv+Hfiltse.*Hfb);
% reflexive impedance (X=>A) [Eq.5]
H.Hxa=Hfiltse.*(Hms-Hce.*Hgto).*Hfiltgto;
% adding the contact dynamics
if (isempty(p.bc)~=1)&&(isempty(p.kc)~=1)
    Hc=p.bc*s+p.kc; %[Eq.A14]
    H.Hxa=H.Hxa.*( (H.Hfx.*Hc)./(1+Hc.*H.Hfx) ); %[Eq.A16]
    H.Hfx=H.Hfx+(1./Hc); %[Eq.A15]
end
% total admittance [Eq.7]
H.Hdx=(H.Hfx)./(He_inv.*H.Hfx+1);

% Stability, open-loop transfer function
Hint=1./(Hi_inv+1e-10); % +1e-10: trick to prevent 'divide by zero'
He=1./(He_inv+1e-10);
% Hol human [Eq.B5]
H.Holh=Hms.*Hfiltse.*Hint.*Hfiltgto.*Hact./(1+Hfiltse.*Hint.*Hfiltgto.*Hce);
% Hol total [Eq.B6]
if (isempty(p.bc)~=1)&&(isempty(p.kc)~=1)
    Hint2=(Hint-Hc.*He.*Hint)./(1-Hc.*He-Hc.*Hint);
else
    Hint2=1./(Hi_inv+He_inv+1e-10);
end
H.Holt=Hms.*Hfiltse.*Hint2.*Hfiltgto.*Hact./(1+Hfiltse.*Hint2.*Hfiltgto.*Hce);

% check for unstability
H.unsh=0;
% unstability can only occur if real(H) becomes smaller than -1 (Nyquist!)
if min(real(H.Holh))<-1
    % solve the exception, i.e. starting on -real
    if (angle(H.Holh(1))>0.95*pi)||(angle(H.Holh(1))<-0.95*pi)
        H180=interp1(imag(H.Holh(1:4)),real(H.Holh(1:4)),0,[],'extrap');
        H.unsh=H.unsh+(H180<-1); % add 1 if unstability present
    end
    % normal case: find where H goes through -real
    a=sign(angle(H.Holh));
    nsign=find(a(1:end-1)~=a(2:end));
    if ~isempty(nsign)
        for loop=1:length(nsign)
            x=imag(H.Holh(nsign(loop):nsign(loop)+1));
            y=real(H.Holh(nsign(loop):nsign(loop)+1));
            H180=interp1(x,y,0);
            H.unsh=H.unsh+(H180<-1); % add 1 if unstability present
        end
    end
end
H.unst=0;
% unstability can only occur if real(H) becomes smaller than -1 (Nyquist!)
if min(real(H.Holt))<-1
    % solve the exception, i.e. starting on -real
    if (angle(H.Holt(1))>0.95*pi)||(angle(H.Holt(1))<-0.95*pi)
        H180=interp1(imag(H.Holt(1:4)),real(H.Holt(1:4)),0,[],'extrap');
        H.unst=H.unst+(H180<-1); % add 1 if unstability present
    end
    % normal case: find where H goes through -real
    a=sign(angle(H.Holt));
    nsign=find(a(1:end-1)~=a(2:end));
    if ~isempty(nsign)
        for loop=1:length(nsign)
            x=imag(H.Holt(nsign(loop):nsign(loop)+1));
            y=real(H.Holt(nsign(loop):nsign(loop)+1));
            H180=interp1(x,y,0);
            H.unst=H.unst+(H180<-1); % add 1 if unstability present
        end
    end
end

% performance criteria [Eq.9]
if H.unsh==0
    J.xh=sum(abs(H.Hfx.^2));
else
    J.xh=NaN;
end
if H.unst==0
    J.xt=sum(abs(H.Hdx.^2));
else
    J.xt=NaN;
end

%% initpar
function [p,q]=initpar(joint)
% initialize model depended settings
if strcmpi(joint,'shoulder')
    % frequency vector 0.1-20 Hz must be linearly spaced, see Jxx
    % extreme sensitive for lower frequencies: added a few points in
    % the lower and higher range.
    q.fr=[0.0001 0.001 0.01 0.1:0.05:20 50 100 500 1000];
    % freq/time vector for the impulse responsie (via ifft(H))
    q.Fs=1000;
    T=40;N=T*q.Fs;
    q.fr_imp=(0:N/2)/T;
    q.t_imp=(0:N-1)/q.Fs-0.5;
    % axes limits
    q.ax.freq.val=[0.5 20];        % limits frequency axes
    q.ax.freq.tick=[0.5 1:10 20];  % ticks
    q.ax.freq.label=[0.5 1:10 20]; % labels
    q.ax.frf_ma.val=[0.00001 0.01];% limits frf mechanical admittance
    q.ax.frf_ma.ylabel='gain [m/N]';
    q.ax.frf_ma.title='F => X';
    q.ax.frf_ri.val=[1e2 1e4];     % limits frf reflexive impedance
    q.ax.frf_ri.ylabel='gain [N/m]';
    q.ax.frf_ri.title='X => EMG';
    q.ax.time.val=[-0.5 2];        % limits time axes
    q.ax.imp.val=[-0.03 0.03];     % limits impulse response
    q.ax.imp.ylabel='impulse [m/N]';
    q.ax.imp.title='response (F=>X)';
    q.ax.step.val=[-0.001 0.004];  % limits impulse response
    q.ax.step.ylabel='step [m/N]';
    % drawing with the model!
    q.ModName='NMCscheme.png';

    % initialize model parameters
    % .norm   initial value
    % .min    minimum value (of the slider)
    % .max    maximum value (of the slider)
    % .unit   string describing the unit of the parameter
    % .gain   factor, e.g. the contact stiffness is presented in kN/m (gain=1000)
    p.m.norm=2;     p.m.min=1;      p.m.max=3;      p.m.unit='kg';      % joint mass/inertia
    p.bq.norm=0;    p.bq.min=0;     p.bq.max=100;   p.bq.unit='Ns/m';   % passive damping
    p.kq.norm=0;    p.kq.min=0;     p.kq.max=1000;  p.kq.unit='N/m';    % passive stiffness
    p.b.norm=30;    p.b.min=0;      p.b.max=100;    p.b.unit='Ns/m';    % active damping
    p.k.norm=400;   p.k.min=0;      p.k.max=2000;   p.k.unit='N/m';     % active stiffness
    p.kse.norm=10000;p.kse.min=1000;p.kse.max=101000;                   % stiffness series elastic element
    p.kse.gain=1000;p.kse.unit='kN/m'; % show kse in kN/m!
    p.bc.norm=200;  p.bc.min=50;    p.bc.max=550;   p.bc.unit='Ns/m';   % contact damping
    p.kc.norm=12000;p.kc.min=1000;  p.kc.max=41000;                     % contact stiffness
    p.kc.gain=1000; p.kc.unit='kN/m'; % show kg in kN/m!
    p.me.norm=1;    p.me.min=0;     p.me.max=5;     p.me.unit='kg';     % external mass/inertia
    p.be.norm=0;    p.be.min=-30;   p.be.max=200;   p.be.unit='Ns/m';   % external damping
    p.ke.norm=0;    p.ke.min=-600;  p.ke.max=2000;  p.ke.unit='N/m';    % external stiffness
    p.F0.norm=2.5;  p.F0.min=1;     p.F0.max=7;     p.F0.unit='Hz';     % bandwidth activation dynamics
    p.Bt.norm=0.7;  p.Bt.min=0.5;   p.Bt.max=2;     p.Bt.unit='';       % rel. damp. activation dynamics
    p.Td.norm=0.025;p.Td.min=0;     p.Td.max=0.05;                      % neural time delay
    p.Td.gain=0.001;p.Td.unit='ms'; % show Td in ms!
    p.kp.norm=0;    p.kp.min=-500;  p.kp.max=1500;  p.kp.unit='N/m';    % position feedback gain
    p.kv.norm=30;   p.kv.min=-50;   p.kv.max=150;   p.kv.unit='Ns/m';   % velocity feedback gain
    p.ka.norm=0;    p.ka.min=-2;    p.ka.max=6;     p.ka.unit='Ns2/m';  % acceleration feedback gain
    p.kf.norm=0;    p.kf.min=-2.5;  p.kf.max=2.5;   p.kf.unit='';       % force feedback gain
elseif strcmpi(joint,'wrist')
    % frequency vector 0.1-50 Hz must be linearly spaced, see Jxx
    % extreme sensitive for lower frequencies: added a few points in
    % the lower and higher range.
    q.fr=[0.0001 0.001 0.01 0.1:0.1:50 100 500 1000];
    % freq/time vector for the impulse response (via ifft(H))
    q.Fs=1000;
    T=40;N=T*q.Fs;
    q.fr_imp=(0:N/2)/T;
    q.t_imp=(0:N-1)/q.Fs-0.5;
    % axes limits
    q.ax.freq.val=[0.5 50];        % limits frequency axes
    q.ax.freq.tick=[0.1 1:10 50];  % ticks
    q.ax.freq.label=[0.1 1:10 50]; % labels
    q.ax.frf_ma.val=[0.001 1];     % limits frf mechanical admittance
    q.ax.frf_ma.ylabel='gain [rad/Nm]';
    q.ax.frf_ma.title='F => X';
    q.ax.frf_ri.val=[1 1e3];       % limits frf reflexive impedance
    q.ax.frf_ri.ylabel='gain [Nm/rad]';
    q.ax.frf_ri.title='X => EMG';
    q.ax.time.val=[-0.25 1];        % limits time axes
    q.ax.imp.val=[-5 5];           % limits impulse response
    q.ax.imp.ylabel='impulse [rad/Nm]';
    q.ax.imp.title='response (F=>X)';
    q.ax.step.val=[-0.01 0.201];   % limits impulse response
    q.ax.step.ylabel='step [rad/Nm]';
    % drawing with the model!
    q.ModName='NMCscheme.png';

    % initialize model parameters
    % .norm   initial value
    % .min    minimum value (of the slider)
    % .max    maximum value (of the slider)
    % .unit   string describing the unit of the parameter
    % .gain   factor, e.g. the contact stiffness is presented in kN/m (gain=1000)
    p.m.norm=0.002; p.m.min=0.001;  p.m.max=0.003;  p.m.unit='kgm2';    % joint mass/inertia
    p.bq.norm=0;    p.bq.min=0;     p.bq.max=0.1;   p.bq.unit='Nms/rad';% passive damping
    p.kq.norm=0;    p.kq.min=0;     p.kq.max=1;     p.kq.unit='Nm/rad'; % passive stiffness
    p.b.norm=0.1;   p.b.min=0;      p.b.max=1;      p.b.unit='Nms/rad'; % active damping
    p.k.norm=10;    p.k.min=0;      p.k.max=100;    p.k.unit='Nm/rad';  % active stiffness
    p.kse.norm=100; p.kse.min=10;p.kse.max=1010;                        % stiffness series elastic element
    p.kse.gain=1;   p.kse.unit='Nm/rad'; % show kse in kNm/rad!
    p.bc.norm=0.1;  p.bc.min=0.01;  p.bc.max=1.01;  p.bc.unit='Nms/rad';% contact damping
    p.kc.norm=100;  p.kc.min=10;    p.kc.max=1010;                      % contact stiffness
    p.kc.gain=1;    p.kc.unit='Nm/rad'; % show kg in kNm/rad!
    p.me.norm=0.002;p.me.min=0;     p.me.max=0.005; p.me.unit='kgm2';   % external mass/inertia
    p.be.norm=0;    p.be.min=-2;    p.be.max=2;     p.be.unit='Nms/rad';% external damping
    p.ke.norm=0;    p.ke.min=-20;   p.ke.max=20;    p.ke.unit='Nm/rad'; % external stiffness
    p.F0.norm=2.5;  p.F0.min=1;     p.F0.max=7;     p.F0.unit='Hz';     % bandwidth activation dynamics
    p.Bt.norm=0.7;  p.Bt.min=0.5;   p.Bt.max=2;     p.Bt.unit='';       % rel. damp. activation dynamics
    p.Td.norm=0.030;p.Td.min=0;     p.Td.max=0.05;                      % neural time delay
    p.Td.gain=0.001;p.Td.unit='ms'; % show Td in ms!
    p.kp.norm=0;    p.kp.min=-10;   p.kp.max=10;    p.kp.unit='Nm/rad'; % position feedback gain
    p.kv.norm=0.2;  p.kv.min=-1;    p.kv.max=1;     p.kv.unit='Nms/rad';% velocity feedback gain
    p.ka.norm=0;    p.ka.min=-0.1;  p.ka.max=0.1;   p.ka.unit='Nms2/rad';% acceleration feedback gain
    p.kf.norm=0;    p.kf.min=-2.5;  p.kf.max=2.5;   p.kf.unit='';       % force feedback gain
elseif strcmpi(joint,'ankle')
    % frequency vector 0.1-50 Hz must be linearly spaced, see Jxx
    % extreme sensitive for lower frequencies: added a few points in
    % the lower and higher range.
    q.fr=[0.0001 0.001 0.01 0.1:0.1:50 100 500 1000];
    % freq/time vector for the impulse response (via ifft(H))
    q.Fs=1000;
    T=40;N=T*q.Fs;
    q.fr_imp=(0:N/2)/T;
    q.t_imp=(0:N-1)/q.Fs-0.5;
    % axes limits
    q.ax.freq.val=[0.5 50];        % limits frequency axes
    q.ax.freq.tick=[0.1 1:10 50];  % ticks
    q.ax.freq.label=[0.1 1:10 50]; % labels
    q.ax.frf_ma.val=[0.001 1];     % limits frf mechanical admittance
    q.ax.frf_ma.ylabel='gain [rad/Nm]';
    q.ax.frf_ma.title='F => X';
    q.ax.frf_ri.val=[1 1e3];       % limits frf reflexive impedance
    q.ax.frf_ri.ylabel='gain [Nm/rad]';
    q.ax.frf_ri.title='X => EMG';
    q.ax.time.val=[-0.1 0.5];        % limits time axes
    q.ax.imp.val=[-1 1];           % limits impulse response
    q.ax.imp.ylabel='impulse [rad/Nm]';
    q.ax.imp.title='response (F=>X)';
    q.ax.step.val=[-0.01 0.051];   % limits impulse response
    q.ax.step.ylabel='step [rad/Nm]';
    % drawing with the model!
    q.ModName='NMCscheme.png';

    % initialize model parameters
    % .norm   initial value
    % .min    minimum value (of the slider)
    % .max    maximum value (of the slider)
    % .unit   string describing the unit of the parameter
    % .gain   factor, e.g. the contact stiffness is presented in kN/m (gain=1000)
    p.m.norm=0.005; p.m.min=0.001;  p.m.max=0.011;  p.m.unit='kgm2';    % joint mass/inertia
    p.bq.norm=0;    p.bq.min=0;     p.bq.max=1;     p.bq.unit='Nms/rad';% passive damping
    p.kq.norm=0;    p.kq.min=0;     p.kq.max=10;    p.kq.unit='Nm/rad'; % passive stiffness
    p.b.norm=1;     p.b.min=0;      p.b.max=1;      p.b.unit='Nms/rad'; % active damping
    p.k.norm=100;   p.k.min=0;      p.k.max=1000;   p.k.unit='Nm/rad';  % active stiffness
    p.kse.norm=1000;p.kse.min=100;  p.kse.max=10100;                    % stiffness series elastic element
    p.kse.gain=1000;p.kse.unit='kNm/rad'; % show kse in kNm/rad!
    p.bc.norm=100;  p.bc.min=50;    p.bc.max=550;   p.bc.unit='Nms/rad';% contact damping
    p.kc.norm=15000;p.kc.min=1000;  p.kc.max=21000;                     % contact stiffness
    p.kc.gain=1000; p.kc.unit='kNm/rad'; % show kg in kNm/rad!
    p.me.norm=0.002;p.me.min=0;     p.me.max=0.005; p.me.unit='kgm2';   % external mass/inertia
    p.be.norm=0;    p.be.min=-2;    p.be.max=2;     p.be.unit='Nms/rad';% external damping
    p.ke.norm=0;    p.ke.min=-20;   p.ke.max=20;    p.ke.unit='Nm/rad'; % external stiffness
    p.F0.norm=2.5;  p.F0.min=1;     p.F0.max=7;     p.F0.unit='Hz';     % bandwidth activation dynamics
    p.Bt.norm=0.7;  p.Bt.min=0.5;   p.Bt.max=2;     p.Bt.unit='';       % rel. damp. activation dynamics
    p.Td.norm=0.030;p.Td.min=0;     p.Td.max=0.05;                      % neural time delay
    p.Td.gain=0.001;p.Td.unit='ms'; % show Td in ms!
    p.kp.norm=1;    p.kp.min=-10;   p.kp.max=10;    p.kp.unit='Nm/rad'; % position feedback gain
    p.kv.norm=0.2;  p.kv.min=-1;    p.kv.max=1;     p.kv.unit='Nms/rad';% velocity feedback gain
    p.ka.norm=0.004;p.ka.min=-0.1;  p.ka.max=0.1;   p.ka.unit='Nms2/rad';% acceleration feedback gain
    p.kf.norm=0;    p.kf.min=-2.5;  p.kf.max=2.5;   p.kf.unit='';       % force feedback gain
end
